/* Fake Geiger Counter with BLE Beacon on ESP32 by Olivier LANVIN 2022
   Based on Neil Kolban example for IDF: https://github.com/nkolban/esp32-snippets/blob/master/cpp_utils/tests/BLE%20Tests/SampleScan.cpp
   Ported to Arduino ESP32 by Evandro Copercini
*/
#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEScan.h>
#include <BLEAdvertisedDevice.h>
#include <Ticker.h>

#define MIN_THRESHOLD 50 
#define MIN_RSSI -90          
#define MAX_DELAY 5000

const int MAX_DIST = pow(MIN_RSSI,2); //max "distance"
const int scanTime = 4;            //in seconds  
const int LED_PIN = 22;            // onboard led pin (depends on board type)
const int DISPLAY_PIN = 26;        // analog display output pin
const int TONE_OUTPUT_PIN = 27;    // piezo buzzer pin
const int TONE_PWM_CHANNEL = 0;    // analog pwm channel
const int FREQ = 3550;             // resonance=3550 Hz    ADAPT TO YOUR OWN TYPE OF PIEZO

char static beaconName[]= "Radioactive";              // beacon's name to search for
char static detectedBeaconName[]= "XXXXXXXXXXX";
unsigned char static buf[]="XXXXXXXXXXXXXXXXXX";
int rssi=-100;                                 // attenuation (dB)
int distance= MAX_DIST;                               // distance     2200 à 9000
int radVal=0;                                         // radiation value (fictive)
long int start=0;
int detectionThreshold= MIN_THRESHOLD;                // initial threshold: natural radioactivity 

BLEScan* pBLEScan;

class MyAdvertisedDeviceCallbacks: public BLEAdvertisedDeviceCallbacks {
    void onResult(BLEAdvertisedDevice advertisedDevice) {
     String address = (advertisedDevice.getAddress()).toString().c_str();
     ((String)(advertisedDevice.toString().c_str())).getBytes(buf, 18);
     String uuid = advertisedDevice.getServiceDataUUID().toString().c_str();
   for(int i=0;i<12;i++){
    detectedBeaconName[i]=(char)buf[i+6];
   }
    detectedBeaconName[11]='\0';  
    Serial.println(detectedBeaconName);
   if(strcmp(detectedBeaconName, beaconName) == 0){                
        rssi=advertisedDevice.getRSSI();
        start=millis();
        }
   // Serial.print("rssi:");
   // Serial.println(rssi);
      }
    };   
      
void tick() { 
  if((random(distance))<detectionThreshold){           // pseudo-random pulse
  digitalWrite(LED_PIN,LOW);                           // flash led
  ledcWriteTone(TONE_PWM_CHANNEL, FREQ);
  delay(3);
  ledcWriteTone(TONE_PWM_CHANNEL, 0);
  digitalWrite(LED_PIN,HIGH); }
  }

Ticker tickerSet;

void setup() {
  Serial.begin(4800);
  pinMode(LED_PIN,OUTPUT);
  digitalWrite(LED_PIN,HIGH);     // led off (depends on board type)
  pinMode(DISPLAY_PIN,OUTPUT);    // analogic display
  ledcAttachPin(DISPLAY_PIN, 1);  // assign display
  ledcSetup(1, 100, 8);           // 100 Hz PWM, 8-bit resolution
  ledcWrite(1, 0);
  ledcAttachPin(TONE_OUTPUT_PIN, TONE_PWM_CHANNEL); // assign buzzer
  tickerSet.attach_ms(30, tick);                    // minimal delay between pulses
  BLEDevice::init("");
  pBLEScan = BLEDevice::getScan();                  //create new scan
  pBLEScan->setAdvertisedDeviceCallbacks(new MyAdvertisedDeviceCallbacks());
  pBLEScan->setActiveScan(true);                    //active scan uses more power, but get results faster
  pBLEScan->setInterval(100);
  pBLEScan->setWindow(99);                          // less or equal setInterval value
}


void loop() {
 if ((millis()-start)>MAX_DELAY){rssi= MIN_RSSI;start=millis();}        // if no detection, rssi=-90
  BLEScanResults foundDevices = pBLEScan->start(scanTime, false);       // scan
  pBLEScan->clearResults();                                             // delete results fromBLEScan buffer to release memory
  distance=pow(rssi,2);                                                 // squared
  radVal= map(distance, 2000, MAX_DIST, 255, 5);                        // convert into 5-255 range
  if (radVal<0)radVal=5;
  if(radVal>255)radVal=255; 
  Serial.println(radVal);
    if(radVal<=6)detectionThreshold= 50; else detectionThreshold=1000;
  ledcWrite(1, radVal);                                                  // display value
  }
